' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE "SDL.INC"
#INCLUDE "GLU.INC"
#INCLUDE "GDIPUTILS.INC"

$WindowCaption = "NeHe Lesson 17"

GLOBAL TextureHandles() AS DWORD
GLOBAL FontBase AS DWORD
GLOBAL cnt1 AS SINGLE
GLOBAL cnt2 AS SINGLE

SUB glPrint (BYVAL x AS LONG, BYVAL y AS LONG, BYREF szText AS ASCIIZ, BYVAL nSet AS LONG, BYVAL FontBase AS DWORD, BYREF TextureHandle AS DWORD)

   IF nSet > 1 THEN nSet = 1

   glBindTexture %GL_TEXTURE_2D, TextureHandle ' Select our font texture
   glDisable %GL_DEPTH_TEST                    ' Disables depth testing
   glMatrixMode %GL_PROJECTION                 ' Select the projection matrix
   glPushMatrix                                ' Store the projection matrix
   glLoadIdentity                              ' Reset the projection matrix
   glOrtho 0, 640, 0, 480, -1, 1               ' Set up an ortho screen
   glMatrixMode %GL_MODELVIEW                  ' Select the modelview matrix
   glPushMatrix                                ' Store the modelview matrix
   glLoadIdentity                              ' Reset the modelview matrix
   glTranslated x, y, 0                        ' Position the text (0,0 - bottom left)
   glListBase FontBase - 32 + (128 * nSet)     ' Choose the font set (0 or 1)
   glCallLists LEN(szText), %GL_UNSIGNED_BYTE, szText  ' Write the text to the screen
   glMatrixMode %GL_PROJECTION                 ' Select the projection matrix
   glPopMatrix                                 ' Restore the old projection matrix
   glMatrixMode %GL_MODELVIEW                  ' Select the modelview matrix
   glPopMatrix                                 ' Restore the old projection matrix
   glEnable %GL_DEPTH_TEST                     ' Enables depth testing

END SUB

' =======================================================================================
' All the setup goes here
' =======================================================================================
SUB SetupScene ()

   LOCAL hr AS LONG
   LOCAL i AS LONG
   LOCAL cx, cy AS SINGLE
   DIM   TextureHandles(1) AS DWORD
   DIM   TextureWidth(1) AS LONG
   DIM   TextureHeight(1) AS LONG
   DIM   strTextureData(1) AS STRING

   ' Load textures from disk
   hr = GdiPlusLoadTexture("Font.bmp", TextureWidth(0), TextureHeight(0), strTextureData(0), %TRUE)
   hr = GdiPlusLoadTexture("Bumps.bmp", TextureWidth(1), TextureHeight(1), strTextureData(1), %TRUE)

   ' Create two textures
   glGenTextures 2, TextureHandles(0)
   FOR i = 0 TO 1
      glBindTexture %GL_TEXTURE_2D, TextureHandles(i)
      glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
      glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
      glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth(i), TextureHeight(i), 0, _
                   %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData(i))
   NEXT

   ' Create 256 display lists
   FontBase = glGenLists(256)
   ' Select our font texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(0)
   FOR i = 0 TO 255
      ' x position of current character
      cx = (i MOD 16) / 16.0!
      ' y position of current character
      cy = (i \ 16) / 16.0!
      ' Start building a list
      glNewList FontBase + i, %GL_COMPILE
         ' Use a quad for each character
         glBegin %GL_QUADS
            ' Texture coordinate (bottom left)
            glTexCoord2f cx, 1 - cy - 0.0625!
            ' Vertex coordinate (bottom left)
            glVertex2i 0,0
            ' Texture coordinate (bottom right)
            glTexCoord2f cx + 0.0625!, 1 - cy - 0.0625!
            ' Vertex coordinate (bottom right)
            glVertex2i 16, 0
            ' Texture coordinate (top right)
            glTexCoord2f cx + 0.0625!, 1 - cy
            ' Vertex coordinate (top right)
            glVertex2i 16,16
            ' Texture coordinate (top left)
            glTexCoord2f cx, 1 - cy
            ' Vertex coordinate (top left)
            glVertex2i 0, 16
         ' Done building our quad character
         glEnd
         ' Move to the right of the character
         glTranslated 10,0,0
      ' Done building the display list
      glEndList
   NEXT

   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Enables clearing of the depth buffer
   glClearDepth 1.0!
   ' Type of depth test to do
   glDepthFunc %GL_LEQUAL
   ' Select the type of blending
   glBlendFunc %GL_SRC_ALPHA, %GL_ONE
   ' Enables smooth color shading
   glShadeModel %GL_SMOOTH
   ' Enable 2D texture mapping
   glEnable %GL_TEXTURE_2D

END SUB
' =======================================================================================

' =======================================================================================
' Draw the scene
' =======================================================================================
SUB DrawScene (BYVAL nWidth AS LONG, BYVAL nHeight AS LONG)

   LOCAL x_m, y_m, z_m, u_m, v_m AS SINGLE
   LOCAL loop_m AS LONG
   LOCAL xtrans, ztrans, ytrans AS SINGLE
   LOCAL sceneroty AS SINGLE

   ' Clear the screen buffer
   glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

   ' Select and setup the projection matrix
   glMatrixMode %GL_PROJECTION
   glLoadIdentity

   ' Prevent divide by zero making height equal one
   IF nHeight = 0 THEN nHeight = 1

   ' Calculate the aspect ratio of the window
   gluPerspective 45.0!, nWidth/nHeight, 1.0!, 100.0!

   ' Select and setup the modelview matrix
   glMatrixMode %GL_MODELVIEW
   glLoadIdentity

   glBindTexture %GL_TEXTURE_2D, TextureHandles(1)  ' Select our second texture
   glTranslatef 0.0!, 0.0!, -5.0!                   ' Move into the screen 5 units
   glRotatef 45.0!, 0.0!, 0.0!, 1.0!                ' Rotate on the Z axis 45 degrees (clockwise)
   glRotatef cnt1 * 30.0!, 1.0!, 1.0!, 0.0!         ' Rotate on the X & Y axis by cnt1 (left to right)
   glDisable %GL_BLEND                              ' Disable blending before we draw in 3d
   glColor3f 1.0!, 1.0!, 1.0!                       ' Bright white
   glBegin %GL_QUADS                                ' Draw our first texture mapped quad
      glTexCoord2d 0.0!,  0.0!                      ' First texture coordinate
      glVertex2f  -1.0!,  1.0!                      ' First vertex
      glTexCoord2d 1.0!,  0.0!                      ' Second texture coordinate
      glVertex2f   1.0!,  1.0!                      ' Second vertex
      glTexCoord2d 1.0!,  1.0!                      ' Third texture coordinate
      glVertex2f   1.0!, -1.0!                      ' Third vertex
      glTexCoord2d 0.0!,  1.0!                      ' Fourth texture coordinate
      glVertex2f  -1.0!, -1.0!                      ' Fourth vertex
   glEnd                                            ' Done drawing the first quad
   glRotatef 90.0!, 1.0!, 1.0!, 0.0!                ' Rotate on the x & y axis by 90 degrees (left to right)
   glBegin %GL_QUADS                                ' Draw our second texture mapped quad
      glTexCoord2d 0.0!,  0.0!                      ' First texture coordinate
      glVertex2f  -1.0!,  1.0!                      ' First vertex
      glTexCoord2d 1.0!,  0.0!                      ' Second texture coordinate
      glVertex2f   1.0!,  1.0!                      ' Second vertex
      glTexCoord2d 1.0!,  1.0!                      ' Third texture coordinate
      glVertex2f   1.0!, -1.0!                      ' Third vertex
      glTexCoord2d 0.0!,  1.0!                      ' Fourth texture coordinate
      glVertex2f  -1.0!, -1.0!                      ' Fourth vertex
   glEnd                                            ' Done drawing our second quad
   glEnable %GL_BLEND                               ' Enable blending

   ' Reset the view
   glLoadIdentity

   ' Pulsing colors based on text position
   glColor3f 1.0! * COS(cnt1),1.0! * SIN(cnt2), 1.0! - 0.5! * COS(cnt1 + cnt2)
   ' Print GL text to the screen
   glPrint 280 + 250 * COS(cnt1), 235 + 200 * SIN(cnt2), "NeHe", 0, FontBase, TextureHandles(0)

   ' Pulsing colors based on text position
   glColor3f 1.0! * SIN(cnt2), 1.0! - 0.5! * COS(cnt1 + cnt2), 1.0! * COS(cnt1)
   ' Print GL text to the screen
   glPrint 280 + 230 * COS(cnt2), 235 + 200 * SIN(cnt1), "OpenGL", 1, FontBase, TextureHandles(0)

   ' Set color to blue
   glColor3f 0.0!, 0.0!, 1.0!
   ' Print GL text to the screen
   glPrint 240 + 200 * COS((cnt2 + cnt1) / 5), 2, "Giuseppe D'Agata",0, FontBase, TextureHandles(0)

   ' Set color to white
   glColor3f 1.0!, 1.0!, 1.0!
   ' Print GL text to the screen
   glPrint 242 + 200 * COS((cnt2 + cnt1) / 5), 2, "Giuseppe D'Agata",0, FontBase, TextureHandles(0)

   cnt1 = cnt1 + 0.01!           ' Increase the first counter
   cnt2 = cnt2 + 0.0081!         ' Increase the second counter

   ' // Swap buffers
   SDL_GL_SwapBuffers

END SUB
' =======================================================================================

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL pscreen AS SDL_Surface PTR
   LOCAL nWidth, nHeight, bpp AS LONG
   LOCAL lResult, bFullScreen AS LONG
   LOCAL szCaption AS ASCIIZ * 256
   LOCAL dwFlags AS DWORD
   LOCAL t, t0, fps AS DOUBLE
   LOCAL nFrames AS LONG

   szCaption = $WindowCaption   ' Window caption

	' Ask the user which screen mode he prefers
	lResult = MessageBox(%NULL, "Would you like to run in fullscreen mode?", _
             "Start fullScreen?", %MB_YESNOCANCEL OR %MB_ICONQUESTION)
   SELECT CASE lResult
      CASE %IDCANCEL : EXIT FUNCTION
      CASE %IDYES    : bFullScreen = %TRUE
      CASE %IDNO     : bFullScreen = %FALSE
   END SELECT

   ' // Initialize SDL's subsystems - in this case, only video.
   IF SDL_Init(%SDL_INIT_VIDEO) < 0 THEN
      MSGBOX "Unable to init SDL: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Attempt to create a 640x480 window
   nWidth = 640 : nHeight = 480 : bpp = 16
   IF bFullScreen THEN
      dwFlags = %SDL_OPENGL OR %SDL_FULLSCREEN
   ELSE
      dwFlags = %SDL_OPENGL
   END IF
   pscreen = SDL_SetVideoMode(nWidth, nHeight, bpp, dwFlags)
   SDL_WM_SetCaption szCaption, ""

   ' // If we fail, return error.
   IF pscreen = %NULL THEN
      SDL_Quit
      MSGBOX "Unable to set the video mode: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Set up the scene
   SetupScene

   ' // Main loop
   LOCAL done AS LONG
   LOCAL uevent AS SDL_Event
   WHILE done = %FALSE

      ' // Poll for events, and handle the ones we care about.
      WHILE SDL_PollEvent(VARPTR(uevent))
         SELECT CASE uevent.type
            CASE %SDL_KEYUP
               ' // Quit if escape is pressed
               IF uevent.key.keysym.sym = %SDLK_ESCAPE THEN
                  done = %TRUE
                  EXIT LOOP
               END IF
            CASE %SDL_QUIT
               done = %TRUE
               EXIT LOOP
         END SELECT
      WEND

      IF ISFALSE bFullScreen THEN
         ' Get time and mouse position
         t = INT(TIMER)
         ' Calculate and display FPS (frames per second)
         IF t > t0 OR nFrames = 0 THEN
            fps = nFrames \ (t - t0)
            wsprintf szCaption, $WindowCaption & " (%i FPS)", BYVAL fps
            SDL_WM_SetCaption szCaption, szCaption
            t0 = t
            nFrames = 0
         END IF
         nFrames = nFrames + 1
      END IF

      ' // Draw the scene
      DrawScene nWidth, nHeight

   WEND

   ' Deletes the lists
   glDeleteLists FontBase, 256
   ' Delete the textures
   glDeleteTextures(2, TextureHandles(0))

   ' Shut down SDL
   SDL_Quit

END FUNCTION
' =======================================================================================
